#include "hw_sync.hpp"
#include <avr/pgmspace.h>

using namespace Devices::I2c;


bool HwSync::Configure(int32_t f)
{
    return I2c::Configure(f);
}


bool HwSync::Init()
{
    I2c::Init();
    return true;
}


bool HwSync::Start(uint8_t const address, Direction const direction)
{
    SendStartCondition();
    WaitForInt();

    if (GetStatus() != TW_START)
    {
        GoIdle();
        return false;
    }

    SendByte(address | (uint8_t) direction);
    WaitForInt();

    if ((direction == Direction::Transmit && GetStatus() != TW_MT_SLA_ACK) ||
        (direction == Direction::Receive && GetStatus() != TW_MR_SLA_ACK))
    {
        GoIdle();
        return false;
    }

    return true;
}


bool HwSync::Stop()
{
    SendStopCondition();
    GoIdle();
    return true;
}


bool HwSync::Transmit(uint8_t const byte)
{
    SendByte(byte);
    WaitForInt();

    if (GetStatus() != TW_MT_DATA_ACK)
    {
        GoIdle();
        return false;
    }

    return true;
}


bool HwSync::Transmit(uint8_t const size, void const * const buffer)
{
    for (uint8_t i = 0; i < size; ++i)
    {
        SendByte(((uint8_t*) buffer)[i]);
        WaitForInt();

        if (GetStatus() != TW_MT_DATA_ACK)
        {
            GoIdle();
            return false;
        }
    }

    return true;
}


bool HwSync::TransmitProgmem(uint8_t const size, void const * const buffer)
{
    for (uint8_t i = 0; i < size; ++i)
    {
        uint8_t byte = pgm_read_byte(((uint8_t*) buffer) + i);
        SendByte(byte);
        WaitForInt();

        if (GetStatus() != TW_MT_DATA_ACK)
        {
            GoIdle();
            return false;
        }
    }

    return true;
}


bool HwSync::ReceiveAck(uint8_t & byte)
{
    AckByte();
    WaitForInt();
    ReceiveByte(byte);

    if (GetStatus() != TW_MR_DATA_ACK)
    {
        GoIdle();
        return false;
    }

    return true;
}


bool HwSync::ReceiveNack(uint8_t & byte)
{
    NackByte();
    WaitForInt();
    ReceiveByte(byte);

    if (GetStatus() != TW_MR_DATA_NACK)
    {
        GoIdle();
        return false;
    }

    return true;
}


bool HwSync::ReceiveAck(uint8_t const size, void * const buffer)
{
    for (uint8_t i = 0; i < size; ++i)
    {
        if (!ReceiveAck(((uint8_t*) buffer)[i]))
        {
            return false;
        }
    }

    return true;
}


bool HwSync::ReceiveNack(uint8_t const size, void * const buffer)
{
    for (uint8_t i = 0; i < size - 1; ++i)
    {
        if (!ReceiveAck(((uint8_t*) buffer)[i]))
        {
            return false;
        }
    }

    return ReceiveNack(((uint8_t*) buffer)[size - 1]);
}
