#include "measurer.hpp"
#include "board.hpp"
#include <util/atomic.h>


using namespace Voltmeter;


Measurer::Measurer()
    : isResultReady(false)
    , nextResult(Voltmeter::Range::_4V, 3600)
    , sampleIndex(0)
    , pendingRange(Voltmeter::Range::_4V)
    , pendingSamplesN(3600)
    , switchRange(false)
{
}


bool Measurer::HasMeasurement()
{
    return isResultReady;
}


Measurement Measurer::GetMeasurement()
{
    isResultReady = false;
    return currentResult;
}


void Measurer::Configure(uint16_t const samplesN)
{
    pendingSamplesN = samplesN;
}


void Measurer::SetRange(Range const range)
{
    pendingRange = range;
    switchRange = true;
}


void Measurer::AddSample(uint16_t sample)
{
    if (switchRange)
    {
        Board::SetRange(pendingRange);
        nextResult = Measurement(pendingRange, pendingSamplesN);
        sampleIndex = 0;
        switchRange = false;
    }
    else if (sampleIndex != nextResult.samplesN)
    {
        nextResult.sum += sample;
        nextResult.sumOfSquares += (uint32_t) sample * sample;

        if (sample < nextResult.min)
        {
            nextResult.min = sample;
        }

        if (sample > nextResult.max)
        {
            nextResult.max = sample;
        }

        ++sampleIndex;
    }
    else
    {
        isResultReady = true;
        currentResult = nextResult;
        Board::SetRange(pendingRange);
        nextResult = Measurement(pendingRange, pendingSamplesN);
        sampleIndex = 0;
    }
}
